unit adxOLFolderItemsEvents_IMPL;

interface

uses
  SysUtils, ComObj, ComServ, ActiveX, adxAddIn, adxOLFolderItemsEvents_TLB, StdVcl,
  {$IFDEF VER130}
  OleServer,
  {$ENDIF VER130}
  Classes, Outlook2000;

type
  TADX_OLFolderItemsEvents = class(TadxAddin, IADX_OLFolderItemsEvents)
  end;

  TFolderItems = class;

  TAddInModule = class(TadxCOMAddInModule)
    CmdBar: TadxOlExplorerCommandbar;
    procedure CmdBarControls0Change(Sender: TObject);
    procedure CmdBarControls1Click(Sender: TObject);
    procedure adxCOMAddInModuleAddInStartupComplete(Sender: TObject);
    procedure adxCOMAddInModuleAddInBeginShutdown(Sender: TObject);
  private
    FFolderItems: TFolderItems;
    procedure ConnectToFolderByIndex(const AIndex: Integer);
  protected
  public
  end;

{$IFDEF VER130}
  TItemsItemAdd = procedure(ASender: TObject; const Item: IDispatch) of object;
  TItemsItemChange = procedure(ASender: TObject; const Item: IDispatch) of object;

  TItems = class(TOleServer)
  private
    FOnItemAdd: TItemsItemAdd;
    FOnItemChange: TItemsItemChange;
    FOnItemRemove: TNotifyEvent;
    FIntf: _Items;
    function GetDefaultInterface: _Items;
  protected
    procedure InitServerData; override;
    procedure InvokeEvent(DispID: TDispID; var Params: TVariantArray); override;
    function Get_Application: _Application;
    function Get_Class_: OlObjectClass;
    function Get_Session: _NameSpace;
    function Get_Parent: IDispatch;
    function Get_Count: Integer;
    function Get_RawTable: IUnknown;
    function Get_IncludeRecurrences: WordBool;
    procedure Set_IncludeRecurrences(IncludeRecurrences: WordBool);
  public
    procedure Connect; override;
    procedure ConnectTo(svrIntf: _Items);
    procedure Disconnect; override;
    function Item(Index: OleVariant): IDispatch;
    function Add: IDispatch; overload;
    function Add(Type_: OleVariant): IDispatch; overload;
    function Find(const Filter: WideString): IDispatch;
    function FindNext: IDispatch;
    function GetFirst: IDispatch;
    function GetLast: IDispatch;
    function GetNext: IDispatch;
    function GetPrevious: IDispatch;
    procedure Remove(Index: Integer);
    procedure ResetColumns;
    function Restrict(const Filter: WideString): _Items;
    procedure SetColumns(const Columns: WideString);
    procedure Sort(const Property_: WideString); overload;
    procedure Sort(const Property_: WideString; Descending: OleVariant); overload;
    property DefaultInterface: _Items read GetDefaultInterface;
    property Application: _Application read Get_Application;
    property Class_: OlObjectClass read Get_Class_;
    property Session: _NameSpace read Get_Session;
    property Parent: IDispatch read Get_Parent;
    property Count: Integer read Get_Count;
    property RawTable: IUnknown read Get_RawTable;
    property IncludeRecurrences: WordBool read Get_IncludeRecurrences write Set_IncludeRecurrences;
  published
    property OnItemAdd: TItemsItemAdd read FOnItemAdd write FOnItemAdd;
    property OnItemChange: TItemsItemChange read FOnItemChange write FOnItemChange;
    property OnItemRemove: TNotifyEvent read FOnItemRemove write FOnItemRemove;
  end;
{$ENDIF VER130}

  TFolderItems = class(TItems)
  private
    function GetFolder: MAPIFolder;
  protected
    procedure DoItemAdd(ASender: TObject; const Item: IDispatch);
    procedure DoItemChange(ASender: TObject; const Item: IDispatch);
    procedure DoItemRemove(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    procedure ConnectTo(AFolder: MAPIFolder);
    property Folder: MAPIFolder read GetFolder;
  end;

implementation

{$R *.dfm}

uses Dialogs;

{$IFDEF VER130}

{ TItems }

procedure TItems.InitServerData;
const
  CServerData: TServerData = (
    ClassID:   '{00063052-0000-0000-C000-000000000046}';
    IntfIID:   '{00063041-0000-0000-C000-000000000046}';
    EventIID:  '{00063077-0000-0000-C000-000000000046}';
    LicenseKey: nil;
    Version: 500);
begin
  ServerData := @CServerData;
end;

procedure TItems.Connect;
var
  punk: IUnknown;
begin
  if FIntf = nil then begin
    punk := GetServer;
    ConnectEvents(punk);
    FIntf := punk as _Items;
  end;
end;

procedure TItems.ConnectTo(svrIntf: _Items);
begin
  Disconnect;
  FIntf := svrIntf;
  ConnectEvents(FIntf);
end;

procedure TItems.DisConnect;
begin
  if Fintf <> nil then begin
    DisconnectEvents(FIntf);
    FIntf := nil;
  end;
end;

function TItems.GetDefaultInterface: _Items;
begin
  if FIntf = nil then
    Connect;
  Assert(FIntf <> nil, 'DefaultInterface is NULL. Component is not connected to Server. You must call ''Connect'' or ''ConnectTo'' before this operation');
  Result := FIntf;
end;

procedure TItems.InvokeEvent(DispID: TDispID; var Params: TVariantArray);
begin
  case DispID of
    -1: Exit;  // DISPID_UNKNOWN
    61441: if Assigned(FOnItemAdd) then
         FOnItemAdd(Self, Params[0] {const IDispatch});
    61442: if Assigned(FOnItemChange) then
         FOnItemChange(Self, Params[0] {const IDispatch});
    61443: if Assigned(FOnItemRemove) then
         FOnItemRemove(Self);
  end; {case DispID}
end;

function TItems.Get_Application: _Application;
begin
  Result := DefaultInterface.Application;
end;

function TItems.Get_Class_: OlObjectClass;
begin
  Result := DefaultInterface.Class_;
end;

function TItems.Get_Session: _NameSpace;
begin
  Result := DefaultInterface.Session;
end;

function TItems.Get_Parent: IDispatch;
begin
  Result := DefaultInterface.Parent;
end;

function TItems.Get_Count: Integer;
begin
  Result := DefaultInterface.Count;
end;

function TItems.Get_RawTable: IUnknown;
begin
  Result := DefaultInterface.RawTable;
end;

function TItems.Get_IncludeRecurrences: WordBool;
begin
  Result := DefaultInterface.IncludeRecurrences;
end;

procedure TItems.Set_IncludeRecurrences(IncludeRecurrences: WordBool);
begin
  DefaultInterface.Set_IncludeRecurrences(IncludeRecurrences);
end;

function TItems.Item(Index: OleVariant): IDispatch;
begin
  Result := DefaultInterface.Item(Index);
end;

function TItems.Add: IDispatch;
begin
  Result := DefaultInterface.Add(EmptyParam);
end;

function TItems.Add(Type_: OleVariant): IDispatch;
begin
  Result := DefaultInterface.Add(Type_);
end;

function TItems.Find(const Filter: WideString): IDispatch;
begin
  Result := DefaultInterface.Find(Filter);
end;

function TItems.FindNext: IDispatch;
begin
  Result := DefaultInterface.FindNext;
end;

function TItems.GetFirst: IDispatch;
begin
  Result := DefaultInterface.GetFirst;
end;

function TItems.GetLast: IDispatch;
begin
  Result := DefaultInterface.GetLast;
end;

function TItems.GetNext: IDispatch;
begin
  Result := DefaultInterface.GetNext;
end;

function TItems.GetPrevious: IDispatch;
begin
  Result := DefaultInterface.GetPrevious;
end;

procedure TItems.Remove(Index: Integer);
begin
  DefaultInterface.Remove(Index);
end;

procedure TItems.ResetColumns;
begin
  DefaultInterface.ResetColumns;
end;

function TItems.Restrict(const Filter: WideString): _Items;
begin
  Result := DefaultInterface.Restrict(Filter);
end;

procedure TItems.SetColumns(const Columns: WideString);
begin
  DefaultInterface.SetColumns(Columns);
end;

procedure TItems.Sort(const Property_: WideString);
begin
  DefaultInterface.Sort(Property_, EmptyParam);
end;

procedure TItems.Sort(const Property_: WideString; Descending: OleVariant);
begin
  DefaultInterface.Sort(Property_, Descending);
end;
{$ENDIF VER130}

{ TFolderItems }

constructor TFolderItems.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Self.OnItemAdd := DoItemAdd;
  Self.OnItemChange := DoItemChange;
  Self.OnItemRemove := DoItemRemove;
end;

procedure TFolderItems.ConnectTo(AFolder: MAPIFolder);
begin
  if Assigned(AFolder) then
    inherited ConnectTo(AFolder.Items);
end;

function TFolderItems.GetFolder: MAPIFolder;
begin
  Result := nil;
  if Assigned(DefaultInterface) then
    Result := DefaultInterface.Parent as MAPIFolder; 
end;

procedure TFolderItems.DoItemAdd(ASender: TObject; const Item: IDispatch);
begin
  ShowMessage('An Item is added to the "' + Folder.Name + '" folder.');
end;

procedure TFolderItems.DoItemChange(ASender: TObject; const Item: IDispatch);
begin
  ShowMessage('An Item is changed in the "' + Folder.Name + '" folder.');
end;

procedure TFolderItems.DoItemRemove(Sender: TObject);
begin
  ShowMessage('An Item is removed from the "' + Folder.Name + '" folder.');
end;

{ TAddInModule }

procedure TAddInModule.adxCOMAddInModuleAddInStartupComplete(Sender: TObject);
begin
  FFolderItems := TFolderItems.Create(Self);
  if CmdBar.Controls[0].AsComboBox.ListIndex > 0 then
    ConnectToFolderByIndex(CmdBar.Controls[0].AsComboBox.ListIndex);
end;

procedure TAddInModule.adxCOMAddInModuleAddInBeginShutdown(Sender: TObject);
begin
  FFolderItems.Disconnect;
  FFolderItems := nil;
end;

procedure TAddInModule.CmdBarControls0Change(Sender: TObject);
begin
  if CmdBar.Controls[0].AsComboBox.ListIndex > 0 then
    ConnectToFolderByIndex(CmdBar.Controls[0].AsComboBox.ListIndex);
end;

procedure TAddInModule.CmdBarControls1Click(Sender: TObject);
begin
  if CmdBar.Controls[0].AsComboBox.ListIndex > 0 then
    ConnectToFolderByIndex(CmdBar.Controls[0].AsComboBox.ListIndex);
end;

procedure TAddInModule.ConnectToFolderByIndex(const AIndex: Integer);
var
  DefFolder: OlDefaultFolders;
begin
  case AIndex of
    1: DefFolder := olFolderDeletedItems;
    2: DefFolder := olFolderDrafts;
    3: DefFolder := olFolderInbox;
    4: DefFolder := olFolderOutbox;
    5: DefFolder := olFolderSentMail;
    6: DefFolder := olFolderContacts;
    7: DefFolder := olFolderTasks;
    8: DefFolder := olFolderCalendar;
  else
    DefFolder := olFolderInbox;
  end;
  FFolderItems.ConnectTo(OutlookApp.GetNamespace('MAPI').GetDefaultFolder(DefFolder));
end;

initialization
  TadxFactory.Create(ComServer, TADX_OLFolderItemsEvents, CLASS_ADX_OLFolderItemsEvents, TAddInModule);

end.
